/******************************************************************************
*******************************************************************************
**                                                                           **
**      C Swing - A graphical file and directory maintenance utility         **
**                                                                           **
*******************************************************************************
**                                                                           **
**      Original C coding by: S. J. Brown                                    **
**                                                                           **
**      Modifications by many others; see the documentation for a list       **
**                                                                           **
**      Copyright 1994, Harry Flowers and Foteos Macrides                    **
**                                                                           **
**      Terms of Copyright:  This software may be freely distributed         **
**          and used by anyone, providing the distributor provides it        **
**          free of charge or at a price which only covers distribution      **
**          costs.  It may not be sold.  Note that versions prior to         **
**          V3.7.4 were public domain, and are not subject to these terms.   **
**                                                                           **
**      Disclaimer: No warranties as to the use or reliability of this       **
**          software are given by the authors or distributors.  Use at       **
**          your own risk.                                                   **
**                                                                           **
*******************************************************************************
******************************************************************************/
 
/*
**  swing_28 output a line to the file manager display
*/
#include <stdio.h>
#include <ssdef.h>
#include <smgdef.h>
#include <smgmsg.h>
#include <fchdef.h>
#include "fatdef.h"
#include "fm2def.h"
#include "more_descrip.h"
#include "swing.h"
#include "swing_refs.h"

#define MIN_FID_SIZE		16
#define MAX_PROT_SIZE   	21
#define GENERAL_DIR_SIZE	35
#define DISPLAY_WIDTH		75

short word_bit[16] ={0x0001, 0x0002, 0x0004, 0x0008,
                     0x0010, 0x0020, 0x0040, 0x0080,
                     0x0100, 0x0200, 0x0400, 0x0800,
                     0x1000, 0x2000, 0x4000, 0x8000};
char prot_bit[] = "RWEDRWEDRWEDRWED";
                     
put_scroll_line(line,fptr,display_type)
int line;
FILER *fptr;
int display_type;
{
    int l, i, max_width, dir_start_pos, status;
    char dir_info[81], output_string[133];

    if (display_type == USER_OPTIONS)
    {
        smg$set_cursor_abs(&displayf, &line, &4);

        status = smg$put_line(&displayf,
                              DSC1(fptr->filename),
                              &0,
                              &fptr->rendition);
        if (status != SS$_NORMAL)
            panic(status,"put line of option data","put scroll line");

        return(TRUE);
    }

    /*
    **  get the directory information if we haven't already got it
    */
    if (fptr->info_lookup == FALSE)
        get_dir_info(fptr);

    i = fptr->info_lookup;

    if (i != SS$_NORMAL)
    {
        /*
        **  couldn't get file info, so show vms error code
        */
        sprintf(dir_info," %s",error_text(i));
        extend(dir_info,GENERAL_DIR_SIZE);
    }
    else
    {
        switch(fm_type)
        {
            /*
            **  case 0 is the default
            */
default:
case 0:
            /*
            **  size/allocation, date dd-mmm-yy, protection
            */
            format_0(dir_info,fptr);
            break;

case 1:
            /*
            **  size/allocation, owner (name), protection
            */
            format_1(dir_info,fptr);
	    break;

case 2: 
            /*
            **  size/allocation, date, protection
            */
            format_2(dir_info,fptr);
            break;

case 3: 
            /*
            **  size/allocation, uic, file id 
            */
            format_3(dir_info,fptr);
            break;

case 4:
            /*
            **  file organisation
            */
            format_4(dir_info,fptr);
            break;

case 5:
            /*
            **  file attributes
            */
            format_5(dir_info,fptr);
            break;

case 6:
            /*
            **  record format
            */
            format_6(dir_info,fptr);
            break;

case 7:
            /*
            **  record attributes
            */
            format_7(dir_info,fptr);
            break;

case 8:
            /*
            **  map area information
            */
            format_8(dir_info,fptr);
            break;

case 9:
            /*
            **  no dir info
            */
            format_9(dir_info,fptr);
            break;
        }
    }

    dir_start_pos = DISPLAY_WIDTH - strlen(dir_info) + 4;
    max_width = DISPLAY_WIDTH - strlen(dir_info) - 1;

    /*
    **  output filename - if too long truncate, put ~ in place of last displayed
    **  character
    */
    strcpy(output_string,fptr->filename);

    if (strlen(output_string) >  max_width)
    {
        output_string[max_width - 1] = '~';
        output_string[max_width] = 0;
    }

    /*
    **  dir info
    */
    status = smg$put_chars(&displayf,DSC1(dir_info),&line,
                           &dir_start_pos,&1);
    if (status != SS$_NORMAL)
        panic(status,"put dir info chars","put scroll line");

    /*
    **  filename
    */
    status = smg$put_chars(&displayf,DSC1(output_string),&line,
                           &4,0,&fptr->rendition);
    if (status != SS$_NORMAL)
        panic(status,"put filename chars","put scroll line");
}

format_0(output,f)
char *output;
FILER *f;
{
    strcpy(output,size_alloc(f));
    strcat(output,short_date(f));
    strcat(output,protection(f));
}

format_1(output,f)
char *output;
FILER *f;
{
    strcpy(output,size_alloc(f));
    strcat(output,owner_name(f));
    strcat(output,protection(f));
}

format_2(output,f)
char *output;
FILER *f;
{
    strcpy(output,size_alloc(f));
    strcat(output,long_date(f));
    strcat(output,protection(f));
}

format_3(output,f)
char *output;
FILER *f;
{
    strcpy(output,size_alloc(f));
    strcat(output,owner_uic(f));
    strcat(output,file_fid(f));
}

format_4(output,f)
char *output;
FILER *f;
{
    switch(f->file_org)
    {
case FAT$C_DIRECT:
        strcpy(output,"Direct    ");
        break;

case FAT$C_INDEXED:
        strcpy(output,"Indexed   ");
        break;

case FAT$C_RELATIVE:
        strcpy(output,"Relative  ");
        break;

case FAT$C_SEQUENTIAL:
        strcpy(output,"Sequential");
        break;

default:
        strcpy(output,"* undefined *");
        break;
    }

    /*
    **  file characteristics
    */
    if (f->uchar & FCH$M_CONTIGB)
        strcat(output," - ctg best try");
    if (f->uchar & FCH$M_CONTIG)
        strcat(output," - contiguous");


    extend(output,GENERAL_DIR_SIZE);
}

format_5(output,f)
char *output;
FILER *f;
{
    int vlm, dvl;
    
    /*
    **  get the file's version limit
    */
    vlm = f->verlim;
    if (vlm == 32767) vlm = 0;

    /*
    **  if it is a directory file, also get and output
    **  its default version limit
    */
    if (strstr(f->filename, ".DIR"))
    {
        dvl = f->versions;
        if (dvl == 32767) dvl = 0;
        sprintf(output,
            "ext %-5d bkt %-3d gbc %-5d vlm %-5d dvl %-5d", 
            f->defext,
            f->bkt,
            f->gbc,
            vlm,
            dvl);
    }
    else
        sprintf(output,
            "ext %-5d bkt %-3d gbc %-5d vlm %-5d", 
            f->defext,
            f->bkt,
            f->gbc,
            vlm);
}

format_6(output,f)
char *output;
FILER *f;
{
    unsigned short size;

    switch(f->rec_type)
    {
case FAT$C_FIXED:
        strcpy(output,"Fixed-length,      ");
        break;

case FAT$C_VARIABLE:
        strcpy(output,"Variable-length,   ");
        break;

case FAT$C_VFC:
        strcpy(output,"Fixed control,     ");
        break;

case FAT$C_UNDEFINED:
        strcpy(output,"Undefined format");
        break;

case FAT$C_STREAM:
        strcpy(output,"RMS stream format");
        break;

case FAT$C_STREAMLF:
        strcpy(output,"Stream with LF");
        break;

case FAT$C_STREAMCR:
        strcpy(output,"Stream with CR");
        break;

default:
        break;
    }

    if (f->rec_type == FAT$C_FIXED ||
        f->rec_type == FAT$C_VARIABLE ||
        f->rec_type == FAT$C_VFC)
    {
        /*
        **  append maximum size of the records
        */
        size = f->rsize;
        if (size == 0 && f->maxrec)
            size = f->maxrec;

        sprintf(_STR(output), "max %-3d bytes", size);
    }

    extend(output,GENERAL_DIR_SIZE);
}

format_7(output,f)
char *output;
FILER *f;
{
    /*
    **  record attributes
    */
 
    strcpy(output,"        -");

    if (f->rec_attr & FAT$M_FORTRANCC)
        strcpy(output,"FORTRAN carriage control");

    if (f->rec_attr & FAT$M_IMPLIEDCC)
        strcpy(output,"Implied carriage control");

    if (f->rec_attr & FAT$M_PRINTCC)
        strcpy(output,"Print file carriage control");

    if (f->rec_attr & FAT$M_NOSPAN)
        strcpy(output,"No spanned records");

    extend(output,GENERAL_DIR_SIZE);
}

format_8(output,f)
char *output;
FILER *f;
{
    output[0] = 0;

    strcpy(output,size_alloc(f));

    sprintf(_STR(output),"hdrs %d  ptrs %-3d ", f->headers, f->fragments);

    if (f->placement& (1 << FM2$V_EXACT))
        strcat(output,"EXACT ");
    else if (f->placement& (1 << FM2$V_ONCYL))
        strcat(output,"CYLDR ");
    else
        strcat(output,"      ");

    if (f->placement& (1 << FM2$V_LBN))
        strcat(output,"LBN");
    else if (f->placement& (1 << FM2$V_RVN))
        strcat(output,"RVN");
    else
        strcat(output,"   ");
}

format_9(output,f)
char *output;
FILER *f;
{
    strcpy(output," ");
}

extend(c,i)
char *c;
int i;
{
    int j;

    for (j = strlen(c); j < i; j++)
        strcat(c," ");
}

size_alloc(f)
FILER *f;
{
    static char _size_alloc[20];

    sprintf(_size_alloc,"%d/%d  ", f->size, f->allocation);
    return(_size_alloc);
}

short_date(f)
FILER *f;
{
    static char _short_date[20];
    int timlen, status;
    char timbuf[40];
   
    /*
    **  date of creation
    */
    timlen = 0;
    status = sys$asctim(&timlen,
                        DSCA1(timbuf),
                        &f->credate[0],
                        0);
    if (status != SS$_NORMAL)
        strcpy(_short_date,"??-???-??  ");
    else
        sprintf(_short_date,"%.7s%c%c  ",timbuf,timbuf[9],timbuf[10]);

    return(_short_date);
}

long_date(f)
FILER *f;
{
    static char _long_date[25];
    int timlen, status;
    char timbuf[40];
   
    /*
    **  date of creation
    */
    timlen = 0;
    status = sys$asctim(&timlen,
                        DSCA1(timbuf),
                        &f->credate[0],
                        0);
    if (status != SS$_NORMAL)
        strcpy(_long_date,"??-???-???? ??:??  ");
    else
        sprintf(_long_date,"%.17s  ",timbuf);

    return(_long_date);
}

protection(f)
FILER *f;
{
    static char _protection[30];
    int i;

    strcpy(_protection,"(");
    for (i = 0; i < 4; i++)
        if ((f->fpro & word_bit[i]) == 0)
            sprintf(_STR(_protection),"%c",prot_bit[i]);

    strcat(_protection,",");
    for (; i < 8; i++)
        if ((f->fpro & word_bit[i]) == 0)
            sprintf(_STR(_protection),"%c",prot_bit[i]);

    strcat(_protection,",");
    for (; i < 12; i++)
        if ((f->fpro & word_bit[i]) == 0)
            sprintf(_STR(_protection),"%c",prot_bit[i]);

    strcat(_protection,",");
    for (; i < 16; i++)
        if ((f->fpro & word_bit[i]) == 0)
            sprintf(_STR(_protection),"%c",prot_bit[i]);
 
    strcat(_protection,")");

    extend(_protection,MAX_PROT_SIZE);
    return(_protection);
}

owner_name(f)
FILER *f;
{
    static _owner_name[30];
    char nambuf[30];
    short namlen;
    int status;

    namlen = 0;
    status = sys$idtoasc(f->uic,
                         &namlen,
                         DSCA1(nambuf),0,0,0);
    if (status == SS$_NORMAL)
    {
        nambuf[namlen] = 0;
        sprintf(_owner_name,"[%s]  ",nambuf);
    }
    else if (status == SS$_NOSUCHID)
    {
        strcpy(_owner_name,owner_uic(f));
    }
    else
    {
        strcpy(_owner_name,"[???]  ");
    }

    if (strlen(_owner_name) < max_owner_width)
        extend(_owner_name,max_owner_width);
    else
        max_owner_width = strlen(_owner_name);

    return(_owner_name);
}

owner_uic(f)
FILER *f;
{
    static char _owner_uic[30];

    sprintf(_owner_uic,"[%05o,%05o]  ",f->grp,f->mbm);
    return(_owner_uic);
}

file_fid(f)
FILER *f;
{
    static char _file_fid[30];

    sprintf(_file_fid,"(%d,%d,%d)",f->fid[0],f->fid[1],f->fid[2]);
    extend(_file_fid,MIN_FID_SIZE);
    return(_file_fid);
}

prot_to_string(prot,string)
unsigned short prot;
char *string;
{
    int i;

    strcpy(string,"(S:");
    for (i = 0; i < 4; i++)
        if ((prot & word_bit[i]) == 0)
            sprintf(_STR(string),"%c",prot_bit[i]);

    strcat(string,",O:");
    for (; i < 8; i++)
        if ((prot & word_bit[i]) == 0)
            sprintf(_STR(string),"%c",prot_bit[i]);

    strcat(string,",G:");
    for (; i < 12; i++)
        if ((prot & word_bit[i]) == 0)
            sprintf(_STR(string),"%c",prot_bit[i]);

    strcat(string,",W:");
    for (; i < 16; i++)
        if ((prot & word_bit[i]) == 0)
            sprintf(_STR(string),"%c",prot_bit[i]);
 
    strcat(string,")");
}

